use reqwest::Method;
use serde::{Deserialize, Serialize};

use crate::types::{AppId, Pot, PotId};

use super::UbisyncRequest;

#[derive(Serialize, Deserialize)]
pub struct AppRegisterRequest {
    pub name: String,
    pub description: String,
    pub app_type: String,
}

#[derive(Serialize, Deserialize)]
pub struct AppRegisterResponse {
    pub token: String,
    pub app_id: AppId,
}

impl UbisyncRequest for AppRegisterRequest {
    type PathParameters = ();
    type Response = AppRegisterResponse;

    fn method(&self) -> reqwest::Method {
        Method::PUT
    }
    fn path(&self, _: Self::PathParameters) -> String {
        "/app/register".to_string()
    }
}

#[derive(Serialize, Deserialize)]
pub struct AppCreatePotRequest {
    pub app_type: Option<String>,
}

#[derive(Serialize, Deserialize)]
pub struct AppCreatePotResponse {
    pub pot_id: PotId,
}

impl UbisyncRequest for AppCreatePotRequest {
    type PathParameters = ();
    type Response = AppCreatePotResponse;

    fn method(&self) -> Method {
        Method::PUT
    }

    fn path(&self, _: Self::PathParameters) -> String {
        "/app/pot".to_string()
    }
}

#[derive(Serialize, Deserialize)]
pub struct AppSetDefaultPotRequest {
    pub pot_id: PotId,
}

#[derive(Serialize, Deserialize)]
pub struct AppSetDefaultPotResponse;

impl UbisyncRequest for AppSetDefaultPotRequest {
    type PathParameters = ();
    type Response = AppSetDefaultPotResponse;

    fn method(&self) -> Method {
        Method::POST
    }

    fn path(&self, _: Self::PathParameters) -> String {
        "/app/pot/default".to_string()
    }
}

#[derive(Serialize, Deserialize)]
pub struct AppGetDefaultPotRequest;

#[derive(Serialize, Deserialize)]
pub struct AppGetDefaultPotResponse {
    pub pot: Pot,
}

impl UbisyncRequest for AppGetDefaultPotRequest {
    type PathParameters = ();
    type Response = AppGetDefaultPotResponse;

    fn method(&self) -> Method {
        Method::GET
    }

    fn path(&self, _: Self::PathParameters) -> String {
        "/app/pot/default".to_string()
    }
}