use reqwest::Method;
use serde::{Deserialize, Serialize};

use crate::types::{ContentUpdateStrategy, Element, ElementContent, ElementId, PotId};

use super::UbisyncRequest;

#[derive(Serialize, Deserialize)]
pub struct ElementCreateRequest {
    pub content: ElementContent,
    pub pot: Option<PotId>,
    pub update_strategy: Option<ContentUpdateStrategy>,
}

#[derive(Serialize, Deserialize)]
pub struct ElementCreateResponse {
    pub id: ElementId,
}

impl UbisyncRequest for ElementCreateRequest {
    type PathParameters = ();
    type Response = ElementCreateResponse;

    fn method(&self) -> Method {
        Method::PUT
    }
    fn path(&self, _: Self::PathParameters) -> String {
        "/element".to_string()
    }
}

#[derive(Serialize, Deserialize)]
pub struct ElementGetRequest;

#[derive(Serialize, Deserialize)]
pub struct ElementGetResponse {
    pub element: Element,
}

impl UbisyncRequest for ElementGetRequest {
    type PathParameters = ElementId;
    type Response = ElementGetResponse;

    fn method(&self) -> Method {
        Method::GET
    }
    fn path(&self, params: Self::PathParameters) -> String {
        format!("/element/{}", params.to_string())
    }
}

#[derive(Serialize, Deserialize)]
pub struct ElementSetRequest {
    pub content: ElementContent,
}

#[derive(Serialize, Deserialize)]
pub struct ElementSetResponse;

impl UbisyncRequest for ElementSetRequest {
    type PathParameters = ElementId;
    type Response = ElementSetResponse;

    fn method(&self) -> Method {
        Method::POST
    }
    fn path(&self, params: Self::PathParameters) -> String {
        format!("/element/{}", serde_json::to_string(&params).unwrap())
    }
}

#[derive(Serialize, Deserialize)]
pub struct ElementRemoveRequest;

#[derive(Serialize, Deserialize)]
pub struct ElementRemoveResponse;

impl UbisyncRequest for ElementRemoveRequest {
    type PathParameters = ElementId;
    type Response = ElementRemoveResponse;

    fn method(&self) -> Method {
        Method::DELETE
    }
    fn path(&self, params: Self::PathParameters) -> String {
        format!("/element/{}", serde_json::to_string(&params).unwrap())
    }
}
